@@ -7,16 +7,14 @@ from datetime import datetime as dt
 from getpass import getpass
 from imghdr import what
 from inspect import cleandoc
+from json import dumps, loads
 from logging import DEBUG, INFO, basicConfig, getLogger
 from os import environ
 from os.path import exists
 from pathlib import Path
 from sys import exit, stdout
 
-from aiohttp.web import Application, Response, get, run_app
 from humanize import naturaldelta
-from jinja2 import Environment, FileSystemLoader
-from redis import Redis
 from slixmpp import ClientXMPP
 
 
@@ -150,6 +148,21 @@ class Config:
         """The Jinja template to render."""
         return self.section.get("template", None)
 
+    @property
+    def serve(self):
+        """Turn on the web server."""
+        return self.section.get("serve", None)
+
+    @property
+    def storage(self):
+        """Choice of storage back-end."""
+        return self.section.get("storage", None)
+
+    @property
+    def storage_file(self):
+        """Path to the file based storage back-end."""
+        return self.section.get("storage_file", None)
+
 
 class Bot(ClientXMPP):
     """XMPP bots for humans."""
@@ -169,7 +182,7 @@ class Bot(ClientXMPP):
         self.init_bot()
         self.register_xmpp_event_handlers()
         self.register_xmpp_plugins()
-        self.init_db()
+        self.init_storage()
         self.run()
 
     def parse_arguments(self):
@@ -179,7 +192,7 @@ class Bot(ClientXMPP):
         self.parser.add_argument(
             "-d",
             "--debug",
-            help="Enable verbose debug logs",
+            help="enable verbose debug logs",
             action="store_const",
             dest="log_level",
             const=DEBUG,
@@ -189,54 +202,76 @@ class Bot(ClientXMPP):
             "-a",
             "--account",
             dest="account",
-            help="Account for the bot account",
+            help="account for the bot account",
         )
         self.parser.add_argument(
             "-p",
             "--password",
             dest="password",
-            help="Password for the bot account",
+            help="password for the bot account",
         )
         self.parser.add_argument(
             "-n",
             "--nick",
             dest="nick",
-            help="Nickname for the bot account",
+            help="nickname for the bot account",
         )
         self.parser.add_argument(
-            "-av", "--avatar", dest="avatar", help="Avatar for the bot account"
+            "-av", "--avatar", dest="avatar", help="avatar for the bot account"
         )
         self.parser.add_argument(
             "-ru",
             "--redis-url",
             dest="redis_url",
-            help="Redis storage connection URL",
+            help="redis storage connection URL",
         )
         self.parser.add_argument(
             "-r",
             "--rooms",
             dest="rooms",
             nargs="+",
-            help="Rooms to automatically join",
+            help="rooms to automatically join",
         )
         self.parser.add_argument(
+            "-naj",
             "--no-auto-join",
             default=False,
             action="store_true",
             dest="no_auto_join",
-            help="Disable automatically joining rooms when invited",
+            help="disable automatically joining rooms when invited",
         )
         self.parser.add_argument(
             "-pt",
             "--port",
             dest="port",
-            help="The port to serve from",
+            help="the port to serve from",
         )
         self.parser.add_argument(
             "-t",
             "--template",
             dest="template",
-            help="The template to render",
+            help="the template to render",
+        )
+        self.parser.add_argument(
+            "-s",
+            "--serve",
+            default=False,
+            action="store_true",
+            dest="serve",
+            help="turn on the web server",
+        )
+        self.parser.add_argument(
+            "-st",
+            "--storage",
+            dest="storage",
+            help="choice of storage back-end",
+            choices=("file", "redis"),
+        )
+        self.parser.add_argument(
+            "-stf",
+            "--storage-file",
+            dest="storage_file",
+            help="path to file based storage back-end",
         )
 
         self.args = self.parser.parse_args()
@@ -265,21 +300,46 @@ class Bot(ClientXMPP):
 
     def generate_config_interactively(self):
         """Generate bot configuration."""
-        print("*" * 79)
         print(
-            "Please enter the XMPP user account of your bot. ",
-            "This is often referred to as the JID (Jabbed ID). ",
-            "An example would be echobot@vvvvvvaria.org",
+            "Please enter the XMPP user account of your bot",
+            "This is often referred to as the JID (Jabber ID)",
+            "An example: echobot@vvvvvvaria.org",
+            "See https://xmpp-servers.404.city to make an account",
             sep="\n",
         )
         account = input("Account: ")
 
         print("Please enter the password for your bot account")
         password = getpass("Password: ")
-        print("*" * 79)
+
+        print(
+            "Please enter the nickname for your bot",
+            "Others will use this name to refer to the bot",
+            sep="\n",
+        )
+        nick = input("Nickname: ")
+
+        print(
+            "Please list the rooms you want your bot to automatically join",
+            "An example: room1@muc.example.com, room2@muc.example.com",
+            "If your bot only responds to direct messages, leave it blank",
+            sep="\n",
+        )
+        rooms = input("Rooms: ")
+
+        if rooms:
+            rooms = [r.strip() for r in rooms.split(",")]
+
+        inputs = {
+            "account": account,
+            "password": password,
+            "nick": nick,
+            "rooms": rooms,
+        }
+        self.log.debug(f"Received {inputs} as input")
 
         config = ConfigParser()
-        config[self.name] = {"account": account, "password": password}
+        config[self.name] = inputs
 
         with open(self.CONFIG_FILE, "w") as file_handle:
             config.write(file_handle)
@@ -333,6 +393,23 @@ class Bot(ClientXMPP):
             or environ.get("XBOT_TEMPLATE", None)
             or "index.html.j2"
         )
+        serve = (
+            self.args.serve
+            or self.config.serve
+            or environ.get("XBOT_SERVE", None)
+        )
+        storage = (
+            self.args.storage
+            or self.config.storage
+            or environ.get("XBOT_STORAGE", None)
+            or "file"
+        )
+        storage_file = (
+            self.args.storage_file
+            or self.config.storage_file
+            or environ.get("XBOT_STORAGE_FILE", None)
+            or f"{nick}.json"
+        )
 
         if not account:
             self.log.error("Unable to discover account")
@@ -354,13 +431,23 @@ class Bot(ClientXMPP):
         self.rooms = rooms
         self.no_auto_join = no_auto_join
         self.port = port
-        self.template = self.load_template(template)
+        self.serve = serve
+        self.template = self.load_template(template) if self.serve else None
+        self.storage = storage
+        self.storage_file = Path(storage_file).absolute()
 
     def load_template(self, template):
         """Load template via Jinja."""
         if not exists(Path(template).absolute()):
             return None
 
+        try:
+            from jinja2 import Environment, FileSystemLoader
+        except ModuleNotFoundError:
+            print("Missing required dependency: jinja2")
+            print("Have you tried `pip install xbotlib[web]`")
+            exit(1)
+
         try:
             loader = FileSystemLoader(searchpath="./")
             env = Environment(loader=loader)
@@ -481,35 +568,69 @@ class Bot(ClientXMPP):
         except AttributeError:
             self.log.info("No additional plugins loaded")
 
-    def init_db(self):
-        """Initialise the Redis key/value store."""
-        if not self.redis_url:
-            self.db = None
-            return self.log.info("No Redis storage discovered")
+    def init_storage(self):
+        """Initialise the storage back-end."""
+        if self.storage == "file":
+            try:
+                self.db = {}
+                if exists(self.storage_file):
+                    self.db = loads(open(self.storage_file, "r").read())
+                self.log.info("Successfully loaded file storage")
+            except Exception as exception:
+                message = f"Failed to load {self.storage_file}: {exception}"
+                self.log.info(message)
+                exit(1)
+        else:
+            try:
+                from redis import Redis
 
-        try:
-            self.db = Redis.from_url(self.redis_url, decode_responses=True)
-            self.log.info("Successfully connected to Redis storage")
-        except ValueError:
-            self.log.info("Failed to connect to Redis storage")
+                self.db = Redis.from_url(self.redis_url, decode_responses=True)
+                return self.log.info("Successfully connected to Redis storage")
+            except ValueError as exception:
+                message = f"Failed to connect to Redis storage: {exception}"
+                self.log.info(message)
+                exit(1)
+            except ModuleNotFoundError:
+                print("missing required dependency using Redis")
+                print("Have you tried `pip install xbotlib[redis]`")
+                exit(1)
 
     def run(self):
         """Run the bot."""
         self.connect()
 
         try:
-            self.serve_web()
+            if self.serve:
+                self.log.info("Turning on the web server")
+                self.serve_web()
             self.process(forever=False)
         except (KeyboardInterrupt, RuntimeError):
-            pass
+            if self.storage != "file":
+                exit(0)
+
+            try:
+                with open(self.storage_file, "w") as handle:
+                    handle.write(dumps(self.db))
+                self.log.info("Successfully saved file storage")
+            except Exception as exception:
+                message = f"Failed to save file storage: {exception}"
+                self.log.info(message)
+                exit(1)
 
     def serve_web(self):
         """Serve the web."""
+        try:
+            from aiohttp.web import Application, get, run_app
+        except ModuleNotFoundError:
+            print("Missing required dependency: aiohttp")
+            print("Have you tried `pip install xbotlib[web]`")
+            exit(1)
+
         self.web = Application()
 
         try:
             self.web.add_routes([get("/", self.serve)])
-        except AttributeError:
+        except Exception:
             self.web.add_routes([get("/", self.default_serve)])
 
         self.log.info(f"Serving on http://0.0.0.0:{self.port}")
@@ -517,6 +638,13 @@ class Bot(ClientXMPP):
 
     async def default_serve(self, request):
         """Default placeholder text for HTML serving."""
+        try:
+            from aiohttp.web import Response
+        except ModuleNotFoundError:
+            print("Missing required dependency: aiohttp")
+            print("Have you tried `pip install xbotlib[web]`")
+            exit(1)
+
         return Response(text=f"{self.nick} is alive and well")
 
     def reply(self, text, to=None, room=None):
@@ -562,4 +690,11 @@ class Bot(ClientXMPP):
 
     def respond(self, response, content_type="text/html"):
         """Send this response back with the web server."""
+        try:
+            from aiohttp.web import Response
+        except ModuleNotFoundError:
+            print("Missing required dependency: aiohttp")
+            print("Have you tried `pip install xbotlib[web]`")
+            exit(1)
+
         return Response(text=response, content_type=content_type)
